//===================================================================
// joypro3.ino
//
// 2019.10.03 cfs1とcfs2とではHATにおけるSWの解釈が異なっていた
// 2019.09.28 HATスイッチの動作を変更
// 2019.09.06 ProMicro基板へ移行、joypro4.inoを変更
// 2019.08.24 Leonardo にて試作
// 2019.08.24 (c)T.Tanaka
//===================================================================
#include <Joystick.h>

Joystick_ Joystick(JOYSTICK_DEFAULT_REPORT_ID, JOYSTICK_TYPE_JOYSTICK, 32, 0,
  true, true, false, true,  true, false, true,   true, false,  false, false);
// XA    YA    Za      Rx     Ry    Rz    Rudd   Throt Accel  Brake  Steering


int	HATmode = 0;	// 0=cfs1,1=cfs2: トグルスイッチをHATに見立てた時の振舞い

unsigned long LEDcount = 0;
unsigned long LEDstate = 0;

const int ADCmin = 0;
const int ADCmax = 1023;
const int ADCcenter = ADCmax / 2;

// Analog port
const int NUM_OF_ADC = 6;	// ATmeg32u4に割当てたアナログポートの数
const int POT_0 = A0;
const int POT_1 = A1;
const int POT_2 = A2;
const int POT_3 = A3;
const int POT_4 = A4;
const int POT_5 = A5;

const int ADCpin[NUM_OF_ADC] = {POT_0, POT_1, POT_2, POT_3, POT_4, POT_5};
int oldAnalog00[NUM_OF_ADC]  = {0, 0, 0, 0, 0, 0};
int lastAnalog00[NUM_OF_ADC] = {0, 0, 0, 0, 0, 0};
int oldAnalog01[NUM_OF_ADC]  = {0, 0, 0, 0, 0, 0};
int lastAnalog01[NUM_OF_ADC] = {0, 0, 0, 0, 0, 0};
int oldAnalog10[NUM_OF_ADC]  = {0, 0, 0, 0, 0, 0};
int lastAnalog10[NUM_OF_ADC] = {0, 0, 0, 0, 0, 0};
int oldAnalog11[NUM_OF_ADC]  = {0, 0, 0, 0, 0, 0};
int lastAnalog11[NUM_OF_ADC] = {0, 0, 0, 0, 0, 0};

int	stick0R = 0;
int	stick1R = 0;
int	potm0R = 0;
int	potm1R = 0;
int	potm2R = 0;
int	potm3R = 0;
int	potm4R = 0;
int	potm5R = 0;
int	sw2R = 0;
int	sw3R = 0;
int	sw4R = 0;
int	buz2R = 0;

int	stick0L = 0;
int	stick1L = 0;
int	potm0L = 0;
int	potm1L = 0;
int	potm2L = 0;
int	potm3L = 0;
int	potm4L = 0;
int	potm5L = 0;
int	sw2L = 0;
int	sw3L = 0;
int	sw4L = 0;
int	buz2L = 0;

const int NUM_OF_POTS = 16;
int potentiometer[NUM_OF_POTS]  = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
//      0       1      2      3      4      5      6      7       8       9     10     11     12     13     14     15
// stick0R,stick1R,potm0R,potm1R,potm2R,potm3R,potm4R,potm5R,stick0L,stick1L,potm0L,potm1L,potm2L,potm3L,potm4L,potm5L


// Buttons(=switch =sw)
const int NUM_OF_BUTTONS = 6;
const int BTN00 = 4;
const int BTN01 = 5;
const int BTN02 = 6;
const int BTN03 = 7;
const int BTN04 = 8;
const int BTN05 = 9;

const int ButtonPin[NUM_OF_BUTTONS] = {BTN00, BTN01, BTN02, BTN03, BTN04, BTN05};
int lastButtonState[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

// for HAT switch
int	sw0RTime = 0;
int	sw1RTime = 0;
int	sw0LTime = 0;
int	sw1LTime = 0;
int	RightIndex = 0;
int	RightIndexold = 0;
int	RightTime = 0;
int	LeftIndex = 0;
int	LeftIndexold = 0;
int	LeftTime = 0;
int	UpRightIndex = 0;
int	UpRightIndexold = 0;
int	UpRightTime = 0;
int	UpLeftIndex = 0;
int	UpLeftIndexold = 0;
int	UpLeftTime = 0;
// 前   右前   右   右後、 後、  左後、左、 左前
// B13,  B14,  B15,  B16,  B17,  B18,  B19,  B20
const int SideTurn[8] =     {12,13,14,15,16,17,18,19};
const int RightTable[8] =   {12,13,14,15,16,16,16,16};		// 5bytes
const int LeftTable[8] =    {12,19,18,17,16,16,16,16};		// 5bytes

// 上前 上右前 上右 上右後 上後 上左後 上左 上左前
// B21,  B22,  B23,  B24,  B25,  B26,  B27,  B28
const int UpperTurn[8] =    {20,21,22,23,24,25,26,27};
const int UpRightTable[8] = {12,20,21,22,23,24,24,24};		// 6bytes
const int UpLeftTable[8] =  {12,20,27,26,25,24,24,24};		// 6bytes

uint16_t	SWnow = 0;
uint16_t	SWnew = 0;
uint16_t	SWold0 = 0;
uint16_t	SWold1 = 0;
uint16_t	SWedge = 0;
uint16_t	SWkeep = 0;
uint16_t	SWlevel = 0;

unsigned long NextTickTime = 0;


//===================================================================
void setup() {
	Serial1.begin(38400);

	pinMode(0, INPUT_PULLUP);
	pinMode(13, OUTPUT);

	for (int index = 0; index < NUM_OF_BUTTONS; index++) {
		pinMode(ButtonPin[index], INPUT_PULLUP);
	}

	pinMode(10, OUTPUT);	// TC4052 SELA
	pinMode(11, OUTPUT);	// TC4052 SELB

//-------------------------------------------
// cfsとcfs2では視界のスイッチ解釈が違っていた。
// cfs1 ボタンスイッチONである方向を見て、スイッチOFFでもその方向を見たまま。
// cfs2 ボタンスイッチONである方向を見て、スイッチOFFで前方に戻る。
// HATスイッチとしては、cfs2の方式が本来か。gameの設定で変更できるのか不明。
//-------------------------------------------
	digitalWrite(11, 0);	// B SELB:SELA = 2'b01
	digitalWrite(10, 1);	// A
	HATmode = ( analogRead(ADCpin[0]) < 0x100 )? 0: 1;
	if ( HATmode == 0 ) {	// スロットルスティックが下側で電源ON時
		tone(12,440,300);		// cfs1のHATスイッチ動作
	} else {							// スロットルスティックが上側で電源ON時
		tone(12,880,200);		// cfs2のHATスイッチ動作
		delay(300);
		tone(12,880,300);
	}
	delay(300);

	Joystick.begin();
	Joystick.setXAxis(ADCcenter);
	Joystick.setYAxis(ADCcenter);
	Joystick.setRxAxis(ADCcenter);
	Joystick.setRyAxis(ADCcenter);
	Joystick.setRudder(ADCcenter);
	Joystick.setThrottle(ADCcenter);

}


//===================================================================
void loop() {
	int index,j,temp;
	uint16_t num;
	uint8_t hex;

	if (millis() >= NextTickTime) {
		NextTickTime = millis() + 16;	// 16mSのインターバルで以降の処理をする。


	// USB開放の為に D2のスイッチ入力で判断する
		if (digitalRead(0) == 0) {	// pushSWが押されていたら
			digitalWrite(13, 0);			// LED消灯
			return;										// USBに出力しない
		} else {										// LED点滅でJOYSTICKの動作中を知らせる
			if ( LEDcount ) {
				LEDcount--;
			} else {									// カウント終了時
				LEDstate = !LEDstate;		// 反転
				if ( LEDstate ) {
					digitalWrite(13, 1);	// 点灯
					LEDcount = 20;				// xTickTime
				} else {
					digitalWrite(13, 0);	// 消灯
					LEDcount = 30;				// xTickTime
				}
			}
		}


//----------------------------------------------------
// Analog input
// 1回前の入力と今回の入力の平均を求め、フィルタリングする。
// Analog 入力の不足をTC4052BPのアナログスイッチで補う。
// デジタルポート2ビットで入力セレクト
// SEL   00       01       10      11					SELB(IO11):SELA(IO10),TC4052BPのB:A
// A0: stick0R  stick1R  trim0R  trim1R				stick値はtrim値で増減
// A1: potm0R   potm1R   potm2R  sw2sw3R			sw2,sw3は両端モーメンタリ
// A2: potm3R   potm4R   potm5R  sw4R					potm3,potm4,potm5はCN9の予備
// A3: stick0L  stick1L  trim0L  trim1L				stick値はtrim値で増減
// A4: potm0L   potm1L   potm2L  sw2sw3L			sw2,sw3は両端モーメンタリ
// A5: potm3L   potm4L   potm5L  sw4L					potm3,potm4,potm5はCN9の予備
//----------------------------------------------------
		digitalWrite(11, 0);	// B SELB:SELA = 2'b00
		digitalWrite(10, 0);	// A
		for (index = 0; index < NUM_OF_ADC; index++) {
		 oldAnalog00[index] = lastAnalog00[index];
		 lastAnalog00[index] = (analogRead(ADCpin[index]) + lastAnalog00[index]) >> 1;
		}

		digitalWrite(11, 0);	// B SELB:SELA = 2'b01
		digitalWrite(10, 1);	// A
		for (index = 0; index < NUM_OF_ADC; index++) {
		 oldAnalog01[index] = lastAnalog01[index];
		 lastAnalog01[index] = (analogRead(ADCpin[index]) + lastAnalog01[index]) >> 1;
		}

		digitalWrite(11, 1);	// B SELB:SELA = 2'b11
		digitalWrite(10, 1);	// A
		for (index = 0; index < NUM_OF_ADC; index++) {
		 oldAnalog11[index] = lastAnalog11[index];
		 lastAnalog11[index] = (analogRead(ADCpin[index]) + lastAnalog11[index]) >> 1;
		}

		digitalWrite(11, 1);	// B SELB:SELA = 2'b10
		digitalWrite(10, 0);	// A
		for (index = 0; index < NUM_OF_ADC; index++) {
		 oldAnalog10[index] = lastAnalog10[index];
		 lastAnalog10[index] = (analogRead(ADCpin[index]) + lastAnalog10[index]) >> 1;
		}



//----------------------------------------------------
// stickはトリムにより増減する。

// 読み取ったanalog値は0～1023(多分int型は+32767～-32768)
// 1024/4 = 256 これを±128%とする。(1%=4)。よってトリムは28%の増減とみなせば
// 28/256 = 1/9 -> 1/8としてトリム読取り値を1/8にする。
// 1024/8 = 128(0x80),よって64(0x40)以上は加算,64未満は減産
//----------------------------------------------------
		temp = lastAnalog10[0] >> 3;	// trim0R 1/8
		if ( temp >= 64 ) {
			stick0R = lastAnalog00[0] + (temp & 0x003F);
			if ( stick0R > 1023) stick0R = 1023;
		} else {
			stick0R = lastAnalog00[0] - (temp ^ 0x003F);
			if ( stick0R < 0 ) stick0R = 0;
		}
		stick0R -= 13;	// マシン固有の補正
		if ( stick0R < 0 ) stick0R = 0;
//		Hex2ToChar( lastAnalog00[0] );
//		Serial1.write(",");
//		Hex2ToChar( lastAnalog10[0] );
//		Serial1.write(",");
//		Hex2ToChar( stick0R );
//		Serial1.write(",");

		temp = lastAnalog11[0] >> 3;	// trim1R
		if ( temp >= 64 ) {
			stick1R = lastAnalog01[0] + (temp & 0x003F);
			if ( stick1R > 1023) stick1R = 1023;
		} else {
			stick1R = lastAnalog01[0] - (temp ^ 0x003F);
			if ( stick1R < 0 ) stick1R = 0;
		}
		stick1R -= 36;	// マシン固有の補正
		if ( stick1R < 0 ) stick1R = 0;
//		Hex2ToChar( lastAnalog01[0] );
//		Serial1.write(",");
//		Hex2ToChar( lastAnalog11[0] );
//		Serial1.write(",");
//		Hex2ToChar( stick1R );
//		Serial1.write(",");

		temp = lastAnalog10[3] >> 3;	// trim0L
		if ( temp >= 64 ) {
			stick0L = lastAnalog00[3] + (temp & 0x003F);
			if ( stick0L > 1023) stick0L = 1023;
		} else {
			stick0L = lastAnalog00[3] - (temp ^ 0x003F);
			if ( stick0L < 0 ) stick0L = 0;
		}
//		stick0L += 1;	// マシン固有の補正
//		if ( stick0L > 1023) stick0L = 1023;
//		Hex2ToChar( lastAnalog00[3] );
//		Serial1.write(",");
//		Hex2ToChar( lastAnalog10[3] );
//		Serial1.write(",");
//		Hex2ToChar( stick0L );
//		Serial1.write(",");

		temp = lastAnalog11[3] >> 3;	// trim0L
		if ( temp >= 64 ) {
			stick1L = lastAnalog01[3] + (temp & 0x003F);
			if ( stick1L > 1023) stick1L = 1023;
		} else {
			stick1L = lastAnalog01[3] - (temp ^ 0x003F);
			if ( stick1L < 0 ) stick1L = 0;
		}
		stick1L -= 24;	// マシン固有の補正
		if ( stick1L < 0 ) stick1L = 0;
//		Hex2ToChar( lastAnalog01[3] );
//		Serial1.write(",");
//		Hex2ToChar( lastAnalog11[3] );
//		Serial1.write(",");
//		Hex2ToChar( stick1L );
//		Serial1.write(",");


// 配列を変数に移動
		potm0R = lastAnalog00[1];
		potm1R = lastAnalog01[1];
		potm2R = lastAnalog10[1];
// 中央位置でブザーを鳴らす為(右横のレバー式ボリューム)
		if ( (potm2R >= 488 ) & ( potm2R < 536) ) {
			buz2R = 1;
		} else {
			buz2R = 0;
		}

// スイッチにデコード
		if ( lastAnalog11[1] < 64 ) {
			sw2R = 0;	// OFF
			sw3R = 1;	// ON
		} else if ( (lastAnalog11[1] >= 496 ) & ( lastAnalog11[1] < 528) ) {
			sw2R = 1;	// ON
			sw3R = 0;	// OFF
		} else {
			sw2R = 0;	// OFF
			sw3R = 0;	// OFF
		}

// 配列を変数に移動
		potm3R = lastAnalog00[2];
		potm4R = lastAnalog01[2];
		potm5R = lastAnalog10[2];
		if ( lastAnalog11[2] < 256 ) {
			sw4R = 1;	// ON
		} else {
			sw4R = 0;	// OFF
		}

		potm0L = lastAnalog00[4];
		potm1L = lastAnalog01[4];
		potm2L = lastAnalog10[4];
// 中央位置でブザーを鳴らす為(左横のレバー式ボリューム)
		if ( (potm2L >= 488 ) & ( potm2L < 536) ) {
			buz2L = 1;
		} else {
			buz2L = 0;
		}

// スイッチにデコード
		if ( lastAnalog11[4] < 64 ) {
			sw2L = 0;	// OFF
			sw3L = 1;	// ON
		} else if ( (lastAnalog11[4] >= 496 ) & ( lastAnalog11[4] < 528) ) {
			sw2L = 1;	// ON
			sw3L = 0;	// OFF
		} else {
			sw2L = 0;	// OFF
			sw3L = 0;	// OFF
		}

// 配列を変数に移動
		potm3L = lastAnalog00[5];
		potm4L = lastAnalog01[5];
		potm5L = lastAnalog10[5];
		if ( lastAnalog11[5] < 256 ) {
			sw4L = 1;	// ON
		} else {
			sw4L = 0;	// OFF
		}


// button switch input
// sw2sw3R,sw2sw3Lの入力において、sw3側に変化する時に中間値経由の為
// チャタリングが生じる。その為スイッチ入力を一旦ビット列に整形し、
// チャタリング除去を行う。
		SWnow = 0x0000;
		if ( !digitalRead(ButtonPin[0]))	SWnow |= 0x0001;			// sw0R 直接入力port	->0  button1
		if ( !digitalRead(ButtonPin[1]))	SWnow |= 0x0002;			// sw1R 直接入力port	->1  button2
		if ( sw2R )												SWnow |= 0x0004;			// sw2R anlog入力より	->2  button3 HAT CW
		if ( sw3R )												SWnow |= 0x0008;			// sw3R anlog入力より	->3  button4 HAT CW
		if ( sw4R )												SWnow |= 0x0010;			// sw4R anlog入力より	->4  button5
		if ( !digitalRead(ButtonPin[2]))	SWnow |= 0x0020;			// sw5R 直接入力port	->5  button6
		if ( !digitalRead(ButtonPin[3]))	SWnow |= 0x0040;			// sw0L 直接入力port	->6  button7
		if ( !digitalRead(ButtonPin[4]))	SWnow |= 0x0080;			// sw1L 直接入力port	->7  button8
		if ( sw2L )												SWnow |= 0x0100;			// sw2L anlog入力より	->8  button9	HAT CCW
		if ( sw3L )												SWnow |= 0x0200;			// sw3L anlog入力より	->9  button10 HAT CCW
		if ( sw4L )												SWnow |= 0x0400;			// sw4L anlog入力より	->10 button11
		if ( !digitalRead(ButtonPin[5]))	SWnow |= 0x0800;			// sw5L 直接入力port	->11 button12
		if ( buz2R )											SWnow |= 0x1000;			// potm2R中間値 			->12 ---
		if ( buz2L )											SWnow |= 0x2000;			// potm2L中間値				->13 ---

		SWnew  = SWnow & SWold1 & SWold0;		// 今回のレベル
		SWold0 = SWold1;										// 入力を記憶(shift)
		SWold1 = SWnow;											// 入力を記憶(shift)

		SWedge  = SWkeep ^ SWnew;						// 前回レベルと今回レベルでエッジ
		SWkeep  = SWnew;										// 今回レベルを保存
		SWlevel = SWkeep;										// 今回レベル
		//SWpos = SWedge &  SWlevel;				// 今回エッジとレベルで正エッジ
		//SWneg = SWedge & ~SWlevel;				// 今回エッジとレベルで負エッジ


//------------------------------------
// Joystick output
//------------------------------------
		for (int index = 0; index < NUM_OF_POTS; index++) {
			switch( index ) {
				case 0:
					if ( stick0R != potentiometer[index]) {	// stick0R
						Joystick.setXAxis( stick0R );
						potentiometer[index] = stick0R;
					}
				break;

				case 1:
					if ( stick1R != potentiometer[index]) {	// stick1R
						Joystick.setThrottle( stick1R );
						potentiometer[index] = stick1R;
					}
				break;

				case 2:
					if ( potm0R != potentiometer[index]) {	// potm0R
						//Joystick.setRxaxis( potm0R );
						potentiometer[index] = potm0R;
					}
				break;

				case 3:
					if ( potm1R != potentiometer[index]) {	// potm1R
						//Joystick.setXAxis( potm1R );
						potentiometer[index] = potm1R;
					}
				break;

				case 4:
					if ( potm2R != potentiometer[index]) {	// potm2R
						Joystick.setRxAxis( potm2R );
						potentiometer[index] = potm2R;
					}
				break;

				case 5:
					if ( potm3R != potentiometer[index]) {	// potm3R
						//Joystick.setXAxis( potm3R );
						potentiometer[index] = potm3R;
					}
				break;

				case 6:
					if ( potm4R != potentiometer[index]) {	// potm4R
						//Joystick.setXAxis( potm4R );
						potentiometer[index] = potm4R;
					}
				break;

				case 7:
					if ( potm5R != potentiometer[index]) {	// potm5R
						//Joystick.setXAxis( potm5R );
						potentiometer[index] = potm5R;
					}
				break;

				case 8:
					if ( stick0L != potentiometer[index]) {	// stick0L
						Joystick.setRudder( stick0L );
						potentiometer[index] = stick0L;
					}
				break;

				case 9:
					if ( stick1L != potentiometer[index]) {	// stick1L
						Joystick.setYAxis( stick1L );
						potentiometer[index] = stick1L;
					}
				break;

				case 10:
					if ( potm0L != potentiometer[index]) {	// potm0L
						//Joystick.setXAxis( potm0L );
						potentiometer[index] = potm0L;
					}
				break;

				case 11:
					if ( potm1L != potentiometer[index]) {	// potm1L
						//Joystick.setXAxis( potm1L );
						potentiometer[index] = potm1L;
					}
				break;

				case 12:
					if ( potm2L != potentiometer[index]) {	// potm2L
						Joystick.setRyAxis( potm2L );
						potentiometer[index] = potm2L;
					}
				break;

				case 13:
					if ( potm3L != potentiometer[index]) {	// potm3L
						//Joystick.setXAxis( potm3L );
						potentiometer[index] = potm3L;
					}
				break;

				case 14:
					if ( potm4L != potentiometer[index]) {	// potm4L
						//Joystick.setXAxis( potm4L );
						potentiometer[index] = potm4L;
					}
				break;

				case 15:
					if ( potm5L != potentiometer[index]) {	// potm5L
						//Joystick.setXAxis( potm5L );
						potentiometer[index] = potm5L;
					}
				break;

				default:
				break;
			}
		}

		//Hex2ToChar( potm0R );
		//Serial1.write(",");
		//Hex2ToChar( potm0L );
		//Serial1.write( 0x0D );
		///Hex2ToChar( potentiometer[1] );
		///Serial1.write(",");
		//Hex2ToChar( potentiometer[2] );
		//Serial1.write(",");
		//Hex2ToChar( potentiometer[3] );
		//Serial1.write(",");
		///Hex2ToChar( potentiometer[8] );
		///Serial1.write(",");
		///Hex2ToChar( potentiometer[9] );
		//Serial1.write(",");
		//Hex2ToChar( potentiometer[10] );
		//Serial1.write(",");
		//Hex2ToChar( potentiometer[11] );
		///Serial1.write( 0x0D );

//------------------------------------
// button switch output
//------------------------------------
		for (int index = 0; index < 14; index++) {	// 0,,11までのスイッチ状態
			switch ( index ) {
				case 0:	// sw0R HATうしろ
					if( HATmode == 0 ) {	// cfs1の時
						SingleEncode( 16, 12, &sw0RTime, 10);
					} else {							// cfs2の時
						if ( SWedge & 0x0001 ) Joystick.setButton( 16, (SWlevel & 0x0001) );	// Button17
					}
					break;

				case 1:	// sw1R 上
					if( HATmode == 0 ) {	// cfs1の時
						SingleEncode( 20, 12, &sw1RTime, 10);
					} else {							// cfs2の時
						if ( SWedge & 0x0001 ) Joystick.setButton( 20, (SWlevel & 0x0001) );	// Button21
					}
					break;

				case 2:	// sw2R HAT右回り チェック
					if ( potm0R < 0x20 ) {	// gameにボタン番号割当て時
						CWturn( SideTurn, &RightIndex, 8);
					} else {								// 通常
						RotaryEncode( RightTable, &RightIndex, &RightIndexold, &RightTime, (potm0R >> 5), 5);
					}
					break;

				case 3:	// sw3R HAT上右回り チェック
					if ( potm0R < 0x20 ) {	// gameにボタン番号割当て時
						CWturn( UpperTurn, &UpRightIndex, 8);
					} else {								// 通常
						RotaryEncode( UpRightTable, &UpRightIndex, &UpRightIndexold, &UpRightTime, (potm0R >> 5), 6);
					}
					break;

				case 6:	// sw0L HAT右
					if( HATmode == 0 ) {	// cfs1の時
						SingleEncode( 14, 12, &sw0LTime, 10);
					} else {							// cfs2の時
						if ( SWedge & 0x0001 ) Joystick.setButton( 14, (SWlevel & 0x0001) );	// Button17
					}
					break;

				case 7:	// sw1L 左
					if( HATmode == 0 ) {	// cfs1の時
						SingleEncode( 18, 12, &sw1LTime, 10);
					} else {							// cfs2の時
						if ( SWedge & 0x0001 ) Joystick.setButton( 18, (SWlevel & 0x0001) );	// Button21
					}
					//SingleEncode( 18, 12, &sw1LTime, 10);
					break;

				case 8:	// sw2L HAT左回り チェック
					if ( potm0L < 0x20 ) {	// gameにボタン番号割当て時
						CCWturn( SideTurn, &LeftIndex, 8);
					} else {								// 通常
						RotaryEncode( LeftTable, &LeftIndex, &LeftIndexold, &LeftTime, (potm0L >> 5), 5);
					}
					break;

				case 9:	// sw3L HAT上左回り チェック
					if ( potm0L < 0x20 ) {	// gameにボタン番号割当て時
						CCWturn( UpperTurn, &UpLeftIndex, 8);
					} else {								// 通常
						RotaryEncode( UpLeftTable, &UpLeftIndex, &UpLeftIndexold, &UpLeftTime, (potm0L >> 5), 6);
					}
					break;

				case 12:
					if ( SWedge & 0x0001 ) {	// index = 12
						if ( SWlevel & 0x0001 ) {	// potm2R 中間値になった時
							tone(12,440,200);
						}
					}
					break;

				case 13:
					if ( SWedge & 0x0001 ) {
						if ( SWlevel & 0x0001 ) {	// potm2L 中間値になった時
							tone(12,440,200);
						}
					}
				break;

				default:
					if ( SWedge & 0x0001 ) {								// edgeで
						Joystick.setButton( index, (SWlevel & 0x0001) );	// 上記以外素直にSW出力
					}
				break;
			}

			SWedge >>= 1;
			SWlevel >>= 1;
		}


	}	// end of NextTickTime loop
}


//===================================================================
/* ------------------------------------------------------------------
モーメンタリスイッチに任意のボタン番号をあたえる。
HATスイッチである方向を見た後、OFFで正面を見る。等に利用

parameter1: ONの時のSW番号
parameter2: OFFになった時のSW番号
parameter3: SW継続の時間work
parameter4: SW継続の時間+1の値
------------------------------------------------------------------ */
void SingleEncode( int ondir, int offdir, int *time, int tmax )
{
	if ( SWedge & 0x0001 ) {									// edgeで
		if (SWlevel & 0x0001) {									// pos edge = SW ON
			Joystick.setButton( ondir, 1 );				// ON 方向のSW
		} else {
			Joystick.setButton( offdir, 1 );			// OFF 方向のSW
		}
		*time = tmax;
	} else if ( SWlevel & 0x0001 ) {					// ON 継続
		if ( *time ) (*time)--;
		if ( *time == 1 ) Joystick.setButton( ondir, 0 );			// ONの継続終わり
		// check
		//	if ( *time ) {
		//		Hex2ToChar( *time );
		//		Serial1.write( 0x0D );
		//	}
		// check end
	} else {																	// OFF 継続
		if ( *time ) (*time)--;
		if ( *time == 1 ) Joystick.setButton( offdir, 0 );	// ONの継続終わり
	}
}

/* ------------------------------------------------------------------
モーメンタリスイッチがONの間首を振り、後ろに留まる。
OFFになれば反対に戻して正面を見る。
cfs1: 古いSWをOFFにして後、新たなSWをONにする。
cfs2: 古いSWはONのまま,新たなSWを追加でONにする。
------------------------------------------------------------------ */
void RotaryEncode( int *table, int *index, int *indexold, int *time, int tmax, int num )
{
	if ( SWedge & 0x0001 ) {	// edgeで
		if( HATmode == 0 ) {		// cfs1の時
			if ( *index != *indexold ) Joystick.setButton( table[*index], 0 );	// 確実にOFF
		}
		*indexold = *index;			// cfs1,cfs2
		*time = tmax;
		if (SWlevel & 0x0001) {										// pos edge = SW ON
			(*index)++;															// 途中からの始まり考慮(dec中にincとか)
			if ( *index > (num -1) ) *index = num -1;
			if ( *index != *indexold ) Joystick.setButton( table[*index], 1 );	// 新たなSWがON
		} else {																	// neg edge = SW OFF
			if( HATmode == 0 ) {
				if ( *index ) (*index)--;							// 0(前)までデクリメント
				if ( *index != *indexold ) Joystick.setButton( table[*index], 1 );	// 新たなSWがON
			} else {
				while ( *index ) {
					Joystick.setButton( table[*index], 0 );	// すべてのSWをOFF
					(*index)--;															// 0(前)までデクリメント
				}
			}
		}

	} else if ( SWlevel & 0x0001 ) {					// ON 継続
		if ( *time ) {
			(*time)--;
			if( HATmode == 0 ) {
				if ( *time == tmax/2 ) {						// 半分でOFF
					if ( *index != *indexold ) Joystick.setButton( table[*index], 0 );	// ONの継続終わり
				}
			}
		} else {																// 設定時間
			*time = tmax;
			*indexold = *index;
			(*index)++;																// 後へ振り向く
			if ( *index > (num -1)) *index = num -1;	// 後ろに留まる、前回SW OFFしている
			if ( *index != *indexold ) Joystick.setButton( table[*index], 1 );	// ON
		}

	} else {																	// OFF 継続
		if ( *time ) {
			(*time)--;
			if( HATmode == 0 ) {
				if ( *time == tmax/2 ) {							// 半分でOFF
					if ( *index != *indexold ) Joystick.setButton( table[*index], 0 );	// ONの継続終わり
				}
			}
		} else {																// 設定時間
			if( HATmode == 0 ) {
				*time = tmax;
				*indexold = *index;
				if ( *index ) (*index)--;							// 0(前)までデクリメント
				if ( *index != *indexold ) Joystick.setButton( table[*index], 1 );		// ON
			}
		}
	}
}


/* ------------------------------------------------------------------
HATの機能にボタン番号を割り振る。
CW or CCWでswitch ON の度にボタン番号を増減する。
gameの設定画面において,前2関数では割り振りできない為,
正エッジでボタンON,負エッジでボタンOFFとする。
parameter1: 配列アドレス
parameter2: 配列index
parameter3: 配列の要素数
------------------------------------------------------------------ */
void CWturn( int *table, int *index, int num )
{
	if ( SWedge & 0x0001 ) {									// edgeで
		if (SWlevel & 0x0001) {									// pos edge = SW ON
			(*index)++;														// 無限ループ
			if ( *index > (num -1) ) *index = 0;
		}
		Joystick.setButton( table[*index], (SWlevel & 0x0001) );	// エッジの時のみ ON/OFF
		// check
		Hex2ToChar( table );
		Serial1.write(",");
		Hex2ToChar( *index );
		Serial1.write( 0x0D );
		// check end
	}
}


void CCWturn( int *table, int *index, int num )
{
	if ( SWedge & 0x0001 ) {									// edgeで
		if (SWlevel & 0x0001) {									// pos edge = SW ON
			*index = ( *index )? *index -1: *index = num -1;
		}
		Joystick.setButton( table[*index], (SWlevel & 0x0001) );	// エッジの時のみ ON/OFF
	}
}


/* ------------------------------------------------------------------
  16進をAscii charに変換してターミナルへ出力する
------------------------------------------------------------------ */
void Hex2ToChar( uint16_t data )
{
   Serial1.write( HexToAscii(data >> 12) );
   Serial1.write( HexToAscii(data >>  8) );
   Serial1.write( HexToAscii(data >>  4) );
   Serial1.write( HexToAscii(data >>  0) );
}


/* ------------------------------------------------------------------
  16進をAsciiコードに変換する
------------------------------------------------------------------ */
uint8_t HexToAscii( uint8_t num )
{
  uint8_t asc;

    num &= 0x0f;
    if( num <= 9) {     /* '0'～'9' */
      asc = num + '0';
    }
    else {
      asc = num + 'A' - 10;
    }
    return asc;   /* 変換終了 */
}


/* ------------------------------------------------------------------
  ASCII文字を16進に変換する
------------------------------------------------------------------ */
int16_t ToHex( uint8_t asc, uint8_t *num )
{
	if( '0' <= asc && asc <= '9') {				// '0'～'9'
		*num = asc - '0';
	}
	else if( 'A' <= asc && asc <= 'F')  {	// 'A' ～ 'F'
		*num = asc - 'A' + 10;
	}
	else if( 'a' <= asc && asc <= 'f') {	// 'a' ～ 'f'
		*num = asc - 'a' + 10;
	}
	else {
		return 0;	// 16進キャラクターではない
	}
	return 1;		// 変換終了
}
